@@ -4,13 +4,16 @@ use bellframe::{Mask, Row, RowBuf};
 use index_vec::IndexVec;
 
 use super::{Boundary, Result};
-use crate::layout::{BlockIdx, BlockVec, Layout, Link, LinkVec, RowIdx, StartOrEnd};
+use crate::{
+    layout::{BlockIdx, BlockVec, Layout, Link, LinkVec, MethodBlock, RowIdx, StartOrEnd},
+    CallVec,
+};
 
 /// Prefix inserted at the front of every leadwise composition to allow it to be parsed as such
 const LEADWISE_PREFIX: &str = "#";
 
 /// Creates a `Layout` where every course is exactly one lead long.
-pub fn leadwise(methods: &[super::Method]) -> Result<Layout> {
+pub fn leadwise(methods: &[super::Method], calls: &CallVec<super::Call>) -> Result<Layout> {
     super::utils::check_duplicate_shorthands(methods)?;
 
     let stage = methods
@@ -18,23 +21,26 @@ pub fn leadwise(methods: &[super::Method]) -> Result<Layout> {
         .map(|m| m.stage())
         .max()
         .expect("Can't compute stage of 0 methods");
-    let blocks = methods
+    let method_blocks = methods
         .iter()
-        .map(|m| {
-            m.first_lead()
-                .clone_map_annots_with_index(|i, _| (i == 0).then(|| m.shorthand.clone()))
+        .map(|m| MethodBlock {
+            block: m
+                .first_lead()
+                .clone_map_annots_with_index(|i, _| (i == 0).then(|| m.shorthand.clone())),
+            count_range: m.count_range.clone(),
         })
         .collect::<BlockVec<_>>();
 
-    let fixed_bells = super::utils::fixed_bells(methods, stage);
+    let fixed_bells = super::utils::fixed_bells(methods, calls, stage);
     let lead_head_mask = Mask::fix_bells(stage, fixed_bells);
 
     Ok(Layout {
         starts: start_or_ends(&lead_head_mask, methods, Boundary::Start),
         ends: start_or_ends(&lead_head_mask, methods, Boundary::End),
-        links: links(methods, &lead_head_mask),
-        blocks,
+        links: links(methods, calls, &lead_head_mask),
+        method_blocks,
         stage,
+        leadwise: true,
     })
 }
 
@@ -85,7 +91,11 @@ fn start_or_ends<I: index_vec::Idx>(
     locs
 }
 
-fn links(methods: &[super::Method], lead_head_mask: &Mask) -> LinkVec<Link> {
+fn links(
+    methods: &[super::Method],
+    calls: &CallVec<super::Call>,
+    lead_head_mask: &Mask,
+) -> LinkVec<Link> {
     let mut call_starts: Vec<HashMap<&str, Vec<CallStart>>> = Vec::new();
     // Maps each lead label to where calls of that label can **end**
     let mut call_ends: HashMap<&str, Vec<CallEnd>> = HashMap::new();
@@ -120,8 +130,8 @@ fn links(methods: &[super::Method], lead_head_mask: &Mask) -> LinkVec<Link> {
 
     // Place calls between every `call_start` and every `call_end` of that lead label
     let mut links = Vec::new();
-    for (method_idx, method) in methods.iter().enumerate() {
-        for call in &method.calls {
+    for (method_idx, _method) in methods.iter().enumerate() {
+        for (call_idx, call) in calls.iter_enumerated() {
             let label = call.lead_location.as_str();
             let starts = &call_starts[method_idx][label];
             let ends = &call_ends[label];
@@ -138,9 +148,8 @@ fn links(methods: &[super::Method], lead_head_mask: &Mask) -> LinkVec<Link> {
                         ch_mask: lead_head_mask.clone(),
                         ch_transposition: &row_after_call * &end.inv_row,
 
-                        debug_name: call.debug_symbol.clone(),
-                        display_name: format!("[{}]", call.debug_symbol),
-                        weight: call.weight,
+                        call_idx: Some(call_idx),
+                        calling_position: String::new(),
                     });
                     // Plain
                     links.push(plain_link(
@@ -188,9 +197,8 @@ fn plain_link(from: RowIdx, to: RowIdx, ch_mask: &Mask, ch_transposition: RowBuf
         ch_mask: ch_mask.clone(),
         ch_transposition,
 
-        debug_name: "p".to_owned(),
-        display_name: String::new(),
-        weight: 0.0,
+        call_idx: None,
+        calling_position: String::new(),
     }
 }
 
