@@ -1,12 +1,17 @@
 //! Utility code for building [`Layout`]s in common scenarios.
 
-use std::fmt::{Display, Formatter};
+use std::{
+    fmt::{Display, Formatter},
+    ops::Range,
+};
 
 use bellframe::{method::RowAnnot, Bell, Block, Mask, PlaceNot, Row, Stage};
 use itertools::Itertools;
 use serde::Deserialize;
 
-use super::Layout;
+use crate::CallVec;
+
+use super::{Layout, MethodBlock};
 
 pub mod coursewise;
 pub mod leadwise;
@@ -15,6 +20,7 @@ mod utils;
 impl Layout {
     pub fn new(
         methods: Vec<self::Method>,
+        calls: &CallVec<self::Call>,
         splice_style: SpliceStyle,
         part_head: &Row,
         leadwise: Option<bool>,
@@ -29,9 +35,9 @@ impl Layout {
         });
 
         if leadwise {
-            leadwise::leadwise(&methods)
+            leadwise::leadwise(&methods, calls)
         } else {
-            coursewise::coursewise(methods, splice_style)
+            coursewise::coursewise(methods, calls, splice_style)
         }
     }
 }
@@ -188,8 +194,8 @@ pub struct Method {
     inner: bellframe::Method,
     shorthand: String,
 
-    /// Which [`Call`]s can be used within leads of this `Method`
-    calls: Vec<Call>,
+    /// The number of rows of this method must fit within this [`Range`]
+    count_range: Range<usize>,
     /// The indices in which we can start a composition during this `Method`.  `None` means any
     /// index is allowed (provided the CH masks are satisfied).
     ///
@@ -212,9 +218,9 @@ pub struct Method {
 impl Method {
     pub fn new(
         method: bellframe::Method,
-        calls: Vec<Call>,
         ch_masks: Vec<(Mask, Bell)>,
         shorthand: String,
+        count_range: Range<usize>,
         start_indices: Option<&[isize]>,
         end_indices: Option<&[isize]>,
     ) -> Self {
@@ -231,9 +237,9 @@ impl Method {
             end_indices: end_indices.map(convert_indices),
 
             inner: method,
-            shorthand,
 
-            calls,
+            shorthand,
+            count_range,
             ch_masks: ch_masks
                 .into_iter()
                 .flat_map(|(mask, calling_bell)| utils::CourseHeadMask::new(mask, calling_bell))
@@ -248,11 +254,15 @@ impl Method {
         }
     }
 
-    fn block(&self, is_spliced: bool) -> Block<Option<String>> {
-        self.plain_course.clone().map_annots_with_index(|idx, _| {
+    fn course_method_block(&self, is_spliced: bool) -> MethodBlock {
+        let block = self.plain_course.clone().map_annots_with_index(|idx, _| {
             let sub_lead_idx = idx % self.lead_len();
             (sub_lead_idx == 0 && is_spliced).then(|| self.shorthand.clone())
-        })
+        });
+        MethodBlock {
+            block,
+            count_range: self.count_range.clone(),
+        }
     }
 }
 
